-----------------------------------------------------------------------------------------------
-- Client Lua Script for QCGR
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- QCGR Module Definition
-----------------------------------------------------------------------------------------------
local QCGR = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- If you want to modify QCGR to swap any other two abilities then just edit the 4 vars below to the relevant IDs and sprites.
local QCId = 26061
local GRId = 16322
local QCSprite = "IconSprites:Icon_SkillMedic_quantumcascade"
local GRSprite = "IconSprites:Icon_SkillMedic_gammarays"

local InactiveSprite = "BK3:UI_BK3_Holo_Framing_2_Blocker"

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function QCGR:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self

    -- initialize variables here
	self:SetDefaults()
    return o
end

function QCGR:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- QCGR OnLoad
-----------------------------------------------------------------------------------------------
function QCGR:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("QCGR.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- QCGR OnDocLoaded
-----------------------------------------------------------------------------------------------
function QCGR:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "Window", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
	    self.wndMain:Show(true, true)
		self.wndMain:SetAnchorOffsets(self.vars.l,self.vars.t,self.vars.r,self.vars.b)
		if self.vars.bLock == 1 then
			self.wndMain:SetStyle("Moveable",false)
			self.wndMain:SetStyle("Sizable",false)
		end
		self:SkillButtonUpdate()
		Apollo.RegisterEventHandler("AbilityBookChange","OnAbilityChange",self)
		Apollo.RegisterTimerHandler("QCGR_DelayTimer","OnDelayTimer",self)
		Apollo.RegisterSlashCommand("qcgr", "OnSlashCommand", self)
		-- Do additional Addon initialization here
	end
end

-----------------------------------------------------------------------------------------------
-- QCGR Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/qcgr"
function QCGR:OnSlashCommand(strCmd, strArg)
	if strArg == "lock" then
		self.vars.bLock = 1
		self:SetIconLock()
	elseif strArg == "unlock" then
		self.vars.bLock = 0
		self:SetIconLock()		
	elseif strArg == "reset" then
		self:SetDefaults()
	elseif strArg == "toggle" then
		self:ToggleQCGR()
	else
		self:ShowOptionsMenu() -- show the window
	end
end

function QCGR:SetDefaults()
	self.vars = {}
	self.vars.l = 0
	self.vars.t = 0
	self.vars.r = 60
	self.vars.b = 60
	if self.wndMain then
		self.wndMain:SetAnchorOffsets(self.vars.l,self.vars.t,self.vars.r,self.vars.b)
	end
	self.vars.bLock = 0
	self:SetIconLock()
end

function QCGR:ShowOptionsMenu()
	if self.wndOptions then
		self:CloseOptionsMenu()
	end
	self.wndOptions = Apollo.LoadForm(self.xmlDoc,"OptionsMenu",nil,self)
	self:SetIconLock()
end

function QCGR:CloseOptionsMenu()
	if not self.wndOptions then
		return
	end
	self.wndOptions:Destroy()
	self.wndOptions = nil
end

function QCGR:OnWindowMouseDown( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation )
	self:ToggleQCGR()
end

function QCGR:OnAbilityChange()
	self:CreateDelayTimer()
end

function QCGR:CreateDelayTimer()
	Apollo.CreateTimer("QCGR_DelayTimer",1,false)
end

function QCGR:OnDelayTimer()
	self:SkillButtonUpdate()
end

function QCGR:SetIconLock()
	if self.wndMain then
		self.wndMain:SetStyle("Moveable",self.vars.bLock == 0)
		self.wndMain:SetStyle("Sizable",self.vars.bLock == 0)
	end
	if self.wndOptions then
		self.wndOptions:FindChild("LockIconButton"):SetCheck(self.vars.bLock == 1)
	end
end

function QCGR:SkillButtonUpdate()
	local las = ActionSetLib.GetCurrentActionSet()
	local newsprite,skillsfound
	skillsfound = 0
	if las then
		for k,v in pairs(las) do
			if v == QCId then
				newsprite = GRSprite
				skillsfound = skillsfound + 1
			elseif v == GRId then
				newsprite = QCSprite
				skillsfound = skillsfound + 1
			end
		end
	else
		self:CreateDelayTimer()
	end
	if skillsfound ~= 1 then
		self.wndMain:Show(false,false)
		newsprite = InactiveSprite
	else
		self.wndMain:Show(true,false)
	end
	self.wndMain:FindChild("SkillButton"):SetSprite(newsprite)
end

function QCGR:KillCarbineAbilitiesWindow()
	local luaAbilities = Apollo.GetAddon("Abilities")
	if luaAbilities and luaAbilities.tWndRefs and luaAbilities.tWndRefs.wndMain then
		Apollo.GetAddon("Abilities"):OnCloseFinal()
	end
end

function QCGR:ToggleQCGR()
	self:KillCarbineAbilitiesWindow()
	local las = ActionSetLib.GetCurrentActionSet()
	local oldid,newid,oldname,newsprite,skillsfnd,laskey
	skillsfound = 0
	for k,v in pairs(las) do
		if v == QCId then
			oldid = QCId
			newid = GRId
			laskey = k
			newsprite = QCSprite
			skillsfound = skillsfound+1
		elseif v == GRId then
			oldid = GRId
			newid = QCId
			laskey = k
			newsprite = GRSprite
			skillsfound = skillsfound+1
		end
	end
	if skillsfound == 1 then -- Only perform swap if ONE of QC or GR found
		local abs = AbilityBook.GetAbilitiesList()
		local nTier
		for k,v in pairs(abs) do
			if v.nId == oldid then
				nTier = v.nCurrentTier
				break
			end
		end
		AbilityBook.UpdateSpellTier(oldid,1)
		AbilityBook.UpdateSpellTier(newid,nTier)
		las[laskey] = newid
		local tResult = ActionSetLib.RequestActionSetChanges(las)
		if tResult.eResult == ActionSetLib.CodeEnumLimitedActionSetResult.Ok then
			self.wndMain:FindChild("SkillButton"):SetSprite(newsprite)
		end
	end
end

function QCGR:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end
	self.vars.l,self.vars.t,self.vars.r,self.vars.b = self.wndMain:GetAnchorOffsets()
	return self.vars
end

function QCGR:OnRestore(eLevel, tData)
	if tData then
		local tVarNames = {"l","t","r","b","bLock"}
		for _,strVarName in pairs(tVarNames) do
			if tData[strVarName] then
				self.vars[strVarName] = tData[strVarName]
			end
		end
	end
end

---------------------------------------------------------------------------------------------------
-- OptionsMenu Functions
---------------------------------------------------------------------------------------------------

function QCGR:OnLockIconCheck( wndHandler, wndControl, eMouseButton )
	if wndControl:IsChecked() then
		self.vars.bLock = 1
	else
		self.vars.bLock = 0
	end
	self:SetIconLock()
end

function QCGR:OnOptionsMenuCloseButton(wndHandler, wndControl, eMouseButton)
	self:CloseOptionsMenu()
end
-----------------------------------------------------------------------------------------------
-- QCGR Instance
-----------------------------------------------------------------------------------------------
local QCGRInst = QCGR:new()
QCGRInst:Init()
